@@ -3,8 +3,8 @@ import jwt
 
 from python_helper import Constant as c
 from python_helper import log, Function, ObjectHelper, ReflectionHelper, DateTimeHelper, SettingHelper
-import datetime
 
+from python_framework.api.src.converter.static import ConverterStatic
 from python_framework.api.src.constant import ConfigurationKeyConstant
 from python_framework.api.src.util import FlaskUtil
 from python_framework.api.src.constant import JwtConstant
@@ -13,6 +13,7 @@ from python_framework.api.src.service.ExceptionHandler import GlobalException
 from python_framework.api.src.annotation.GlobalExceptionAnnotation import EncapsulateItWithGlobalException
 
 
+OPTION_VERIFY_SIGNATURE = 'verify_signature'
 KEY_API_INSTANCE = 'apiInstance'
 API_INSTANCE_HOLDER = {
     KEY_API_INSTANCE: None
@@ -33,29 +34,32 @@ class JwtManager:
         return jwt.encode(payload, self.secret, algorithm=self.algorithm, headers=headers if ObjectHelper.isNotNone else dict()).decode()
 
     @EncapsulateItWithGlobalException(message=JwtConstant.UNAUTHORIZED_MESSAGE, status=HttpStatus.UNAUTHORIZED)
-    def decode(self, encoded_payload, options=None) :
-        return jwt.decode(encoded_payload, self.secret, algorithms=self.algorithm, options=options if ObjectHelper.isNotNone(options) else dict())
+    def decode(self, encodedPayload, options=None) :
+        return jwt.decode(encodedPayload, self.secret, algorithms=self.algorithm, options=options if ObjectHelper.isNotNone(options) else dict())
 
     @EncapsulateItWithGlobalException(message=JwtConstant.UNAUTHORIZED_MESSAGE, status=HttpStatus.UNAUTHORIZED)
-    def verifyAuthorizaionAccess(self, decodedSessionToken) :
-        if ObjectHelper.isEmpty(decodedSessionToken) or decodedSessionToken[JwtConstant.KW_JTI] in BLACK_LIST:
-            raise Exception('Session already closed')
+    def validateAccessSession(self, rawJwt=None, options=None):
+        decodedSessionToken = self.validateGeneralSessionAndReturnItDecoded(rawJwt=rawJwt, options=options)
+        assert decodedSessionToken.get(JwtConstant.KW_TYPE) == JwtConstant.ACCESS_VALUE_TYPE, f'Access session should have type {JwtConstant.ACCESS_VALUE_TYPE}, but it is {decodedSessionToken.get(JwtConstant.KW_TYPE)}'
 
     @EncapsulateItWithGlobalException(message=JwtConstant.UNAUTHORIZED_MESSAGE, status=HttpStatus.UNAUTHORIZED)
-    def raiseInvalidAccess(self, logMessage) :
-        raise Exception(logMessage)
+    def validateRefreshSession(self, rawJwt=None, options=None):
+        decodedSessionToken = self.validateGeneralSessionAndReturnItDecoded(rawJwt=rawJwt, options=options)
+        assert decodedSessionToken.get(JwtConstant.KW_TYPE) == JwtConstant.REFRESH_VALUE_TYPE, f'Refresh session should have type {JwtConstant.REFRESH_VALUE_TYPE}, but it is {decodedSessionToken.get(JwtConstant.KW_TYPE)}'
 
-    @EncapsulateItWithGlobalException(message=JwtConstant.UNAUTHORIZED_MESSAGE, status=HttpStatus.UNAUTHORIZED)
-    def validateSession(self, rawJwt=None, options=None):
+    def validateGeneralSessionAndReturnItDecoded(self, rawJwt=None, options=None):
         decodedSessionToken = self.getDecodedToken(rawJwt=rawJwt, options=options)
-        self.verifyAuthorizaionAccess(decodedSessionToken)
+        assert ObjectHelper.isDictionary(decodedSessionToken), f'Invalid session type. It should be a dictionary, bu it is {type(decodedSessionToken)}'
+        assert not ObjectHelper.isEmpty(decodedSessionToken), 'Session cannot be empty'
+        assert not decodedSessionToken[JwtConstant.KW_JTI] in BLACK_LIST, f'Session {decodedSessionToken[JwtConstant.KW_JTI]} already closed'
+        return decodedSessionToken
 
     def getBody(self, rawJwt=None, options=None):
         decodedSessionToken = self.getDecodedToken(rawJwt=rawJwt, options=options)
         return decodedSessionToken
 
     def getUnverifiedBody(self, rawJwt=None):
-        return self.getDecodedToken(rawJwt=rawJwt, options={"verify_signature": False})
+        return self.getDecodedToken(rawJwt=rawJwt, options={OPTION_VERIFY_SIGNATURE: False})
 
     def getUnverifiedHeaders(self):
         return jwt.get_unverified_header(self.getEncodedTokenWithoutType())
@@ -66,22 +70,29 @@ class JwtManager:
         return self.decode(self.getEncodedTokenWithoutType(), options=options)
 
     def getEncodedTokenWithoutType(self):
-        encoded_payload = self.captureTokenFromRequestHeader()
-        if ObjectHelper.isNone(encoded_payload):
-            self.raiseInvalidAccess('JWT session token cannot be None')
-        if not encoded_payload.startswith(f'{self.headerType} '):
-            self.raiseInvalidAccess(f'JWT session token must starts with {self.headerType}')
-        return encoded_payload[len(f'{self.headerType} '):].encode()
+        encodedPayload = self.captureTokenFromRequestHeader()
+        assert ObjectHelper.isNone(encodedPayload), 'JWT session token cannot be None'
+        assert encodedPayload.startswith(f'{self.headerType} '), f'JWT session token must starts with {self.headerType}'
+        return encodedPayload[len(f'{self.headerType} '):].encode()
 
     def captureTokenFromRequestHeader(self):
         return FlaskUtil.safellyGetHeaders().get(self.headerName)
 
-
 @Function
-def jwtRequired(function, *args, **kwargs) :
+def jwtAccessRequired(function, *args, **kwargs) :
     def innerFunction(*args, **kwargs) :
         ###- arguments=args[0] --> python weardnes in it's full glory
-        retrieveApiInstance(arguments=args[0]).session.validateSession()
+        retrieveApiInstance(arguments=args[0]).sessionManager.validateAccessSession()
+        functionReturn = function(*args, **kwargs)
+        return functionReturn
+    ReflectionHelper.overrideSignatures(innerFunction, function)
+    return innerFunction
+
+@Function
+def jwtRefreshRequired(function, *args, **kwargs) :
+    def innerFunction(*args, **kwargs) :
+        ###- arguments=args[0] --> python weardnes in it's full glory
+        retrieveApiInstance(arguments=args[0]).sessionManager.validateRefreshSession()
         functionReturn = function(*args, **kwargs)
         return functionReturn
     ReflectionHelper.overrideSignatures(innerFunction, function)
@@ -91,13 +102,13 @@ def jwtRequired(function, *args, **kwargs) :
 def getJwtBody(rawJwt=None, apiInstance=None) :
     # print(f'{getJwtBody}: apiInstance:{apiInstance}')
     if ObjectHelper.isNone(rawJwt):
-        return retrieveApiInstance(apiInstance=apiInstance).session.getBody(rawJwt=rawJwt)
+        return retrieveApiInstance(apiInstance=apiInstance).sessionManager.getBody(rawJwt=rawJwt)
     return rawJwt
 
 @EncapsulateItWithGlobalException(message=JwtConstant.UNAUTHORIZED_MESSAGE, status=HttpStatus.UNAUTHORIZED)
 def getJwtHeaders(apiInstance=None):
     # print(f'{getJwtHeaders}: apiInstance:{apiInstance}')
-    headers = retrieveApiInstance(apiInstance=apiInstance).session.getUnverifiedHeaders()
+    headers = retrieveApiInstance(apiInstance=apiInstance).sessionManager.getUnverifiedHeaders()
     return headers if ObjectHelper.isNotNone(headers) else dict()
 
 @EncapsulateItWithGlobalException(message=JwtConstant.UNAUTHORIZED_MESSAGE, status=HttpStatus.UNAUTHORIZED)
@@ -127,17 +138,18 @@ def addUserToBlackList() :
     BLACK_LIST.add(getJti())
 
 @Function
-def getJwtMannager(
-    appInstance,
-    jwtSecret,
-    algorithm = JwtConstant.DEFAULT_JWT_SESSION_ALGORITHM,
-    headerName = JwtConstant.DEFAULT_JWT_SESSION_HEADER_NAME,
-    headerType = JwtConstant.DEFAULT_JWT_SESSION_HEADER_TYPE
-) :
-    if not jwtSecret :
+def getJwtMannager(appInstance, jwtSecret, algorithm=None, headerName=None, headerType=None):
+    if SettingHelper.activeEnvironmentIsLocal():
+        log.setting(getJwtMannager, f'JWT secret: {jwtSecret}')
+    if not jwtSecret:
         log.warning(getJwtMannager, f'Not possible to instanciate sessionManager{c.DOT_SPACE_CAUSE}Missing jwt secret at {ConfigurationKeyConstant.API_SESSION_SECRET}')
-    else :
-        jwtMannager = JwtManager(jwtSecret, algorithm, headerName, headerType)
+    else:
+        jwtMannager = JwtManager(
+            jwtSecret,
+            ConverterStatic.getValueOrDefault(algorithm, JwtConstant.DEFAULT_JWT_SESSION_ALGORITHM),
+            ConverterStatic.getValueOrDefault(headerName, JwtConstant.DEFAULT_JWT_SESSION_HEADER_NAME),
+            ConverterStatic.getValueOrDefault(headerType, JwtConstant.DEFAULT_JWT_SESSION_HEADER_TYPE)
+        )
         return jwtMannager
 
 @Function
@@ -147,9 +159,9 @@ def addJwt(jwtInstance) :
 @EncapsulateItWithGlobalException(message=JwtConstant.UNAUTHORIZED_MESSAGE, status=HttpStatus.UNAUTHORIZED)
 def createAccessToken(identity, contextList, deltaMinutes=None, headers=None, data=None, apiInstance=None):
     if deltaMinutes :
-        deltaMinutes = datetime.timedelta(minutes=deltaMinutes)
+        deltaMinutes = DateTimeHelper.timeDelta(minutes=deltaMinutes)
     timeNow = DateTimeHelper.dateTimeNow()
-    return retrieveApiInstance(apiInstance=apiInstance).session.encode({
+    return retrieveApiInstance(apiInstance=apiInstance).sessionManager.encode({
             JwtConstant.KW_IAT: timeNow,
             JwtConstant.KW_NFB: timeNow,
             JwtConstant.KW_JTI: f"{int(f'{time.time()}'.replace('.', ''))+int(f'{time.time()}'.replace('.', ''))}",
@@ -158,8 +170,8 @@ def createAccessToken(identity, contextList, deltaMinutes=None, headers=None, da
             JwtConstant.KW_FRESH: False,
             JwtConstant.KW_TYPE: JwtConstant.ACCESS_VALUE_TYPE,
             JwtConstant.KW_CLAIMS: {
-                JwtConstant.KW_CONTEXT: contextList,
-                JwtConstant.KW_DATA: data
+                JwtConstant.KW_CONTEXT: contextList if ObjectHelper.isList(contextList) else [] if ObjectHelper.isNone(contextList) else raiseSessionContextCannotBeNone(),
+                JwtConstant.KW_DATA: data if ObjectHelper.isNotNone(data) else {}
             }
         },
         headers = headers
@@ -168,9 +180,9 @@ def createAccessToken(identity, contextList, deltaMinutes=None, headers=None, da
 @EncapsulateItWithGlobalException(message=JwtConstant.UNAUTHORIZED_MESSAGE, status=HttpStatus.UNAUTHORIZED)
 def refreshAccessToken(identity, contextList, deltaMinutes=None, headers=None, data=None, apiInstance=None):
     if ObjectHelper.isNotNone(deltaMinutes) :
-        deltaMinutes = datetime.timedelta(minutes=deltaMinutes)
+        deltaMinutes = DateTimeHelper.timeDelta(minutes=deltaMinutes)
     timeNow = DateTimeHelper.dateTimeNow()
-    return retrieveApiInstance(apiInstance=apiInstance).session.encode({
+    return retrieveApiInstance(apiInstance=apiInstance).sessionManager.encode({
             JwtConstant.KW_IAT: timeNow,
             JwtConstant.KW_NFB: timeNow,
             JwtConstant.KW_JTI: f"{int(f'{time.time()}'.replace('.', ''))+int(f'{time.time()}'.replace('.', ''))}",
@@ -179,8 +191,8 @@ def refreshAccessToken(identity, contextList, deltaMinutes=None, headers=None, d
             JwtConstant.KW_FRESH: False,
             JwtConstant.KW_TYPE: JwtConstant.REFRESH_VALUE_TYPE,
             JwtConstant.KW_CLAIMS: {
-                JwtConstant.KW_CONTEXT: contextList,
-                JwtConstant.KW_DATA: data
+                JwtConstant.KW_CONTEXT: contextList if ObjectHelper.isList(contextList) else [] if ObjectHelper.isNone(contextList) else raiseSessionContextCannotBeNone(),
+                JwtConstant.KW_DATA: data if ObjectHelper.isNotNone(data) else {}
             }
         },
         headers = headers
@@ -192,7 +204,7 @@ def patchAccessToken(newContextList=None, headers=None, data=None, apiInstance=N
     # expiresDelta=rawJwt.get(JwtConstant.KW_EXPIRATION)
     print(time.time())
     timeNow = DateTimeHelper.dateTimeNow()
-    deltaMinutes = datetime.timedelta(minutes=1)
+    deltaMinutes = DateTimeHelper.timeDelta(minutes=1)
     userClaims = {
         JwtConstant.KW_CONTEXT: list(set([
             *getContext(rawJwt=rawJwt),
@@ -208,14 +220,14 @@ def patchAccessToken(newContextList=None, headers=None, data=None, apiInstance=N
         }
     }
     apiInstance = retrieveApiInstance(apiInstance=apiInstance)
-    return apiInstance.session.encode({
+    return apiInstance.sessionManager.encode({
             JwtConstant.KW_IAT: timeNow,
             JwtConstant.KW_NFB: timeNow,
             JwtConstant.KW_JTI: f"{int(f'{time.time()}'.replace('.', ''))+int(f'{time.time()}'.replace('.', ''))}",
             JwtConstant.KW_EXPIRATION: timeNow + deltaMinutes,
             JwtConstant.KW_IDENTITY: getIdentity(rawJwt=rawJwt, apiInstance=apiInstance),
             JwtConstant.KW_FRESH: False,
-            JwtConstant.KW_TYPE: JwtConstant.REFRESH_VALUE_TYPE,
+            JwtConstant.KW_TYPE: JwtConstant.ACCESS_VALUE_TYPE,
             JwtConstant.KW_CLAIMS: userClaims
         },
         headers = headers
@@ -245,13 +257,16 @@ def getCurrentSession(sessionClass=None, apiInstance=None):
                 ReflectionHelper.setAttributeOrMethod(currentSession, attributeName, data.get(attributeName))
         return currentSession
 
-def addSession(api, app):
-    sessionKey = api.globals.getApiSetting(ConfigurationKeyConstant.API_SESSION_SECRET)
-    if SettingHelper.activeEnvironmentIsLocal():
-        log.setting(addSession, f'JWT secret: {sessionKey}')
-    api.session = getJwtMannager(app, sessionKey)
+def addSessionManager(apiInstance, appInstance):
     try:
-        api.session.api = api
+        apiInstance.sessionManager = getJwtMannager(
+            appInstance,
+            apiInstance.globals.getApiSetting(ConfigurationKeyConstant.API_SESSION_SECRET),
+            algorithm = apiInstance.globals.getApiSetting(ConfigurationKeyConstant.API_SESSION_ALGORITHM),
+            headerName = apiInstance.globals.getApiSetting(ConfigurationKeyConstant.API_SESSION_HEADER),
+            headerType = apiInstance.globals.getApiSetting(ConfigurationKeyConstant.API_SESSION_TYPE)
+        )
+        apiInstance.sessionManager.api = apiInstance
     except Exception as exception:
         log.warning(addSession, 'Not possible to add Session Manager', exception=exception)
 
@@ -271,10 +286,13 @@ def retrieveApiInstance(apiInstance=None, arguments=None):
         apiInstance = FlaskUtil.getApi()
     if ObjectHelper.isNone(apiInstance):
         raiseUnretrievedApiInstance()
-    if ObjectHelper.isNone(apiInstance.session):
+    if ObjectHelper.isNone(apiInstance.sessionManager):
         raise Exception('There is no session manager')
     API_INSTANCE_HOLDER[KEY_API_INSTANCE] = apiInstance
     return apiInstance
 
+def raiseSessionContextCannotBeNone():
+    raise Exception('Context cannot be None')
+
 def raiseUnretrievedApiInstance():
     raise Exception('Not possible to retrieve api instance')
